; ProcessRaw
;
; input: directory 		=> string containing directory path of raw data file (fid file)
;		 scanNumString  => string representing the bruker scan number
;		 matrixSize		=> array holding # of columns (frequency encodes, RE and IM) and # of rows
;					       (phase encodes) used to acquire image
;		 nImages 		=> # of images acquired during scan
;		 dcOffset		=> array containing dc offset [RE,IM] applied during image acquisition
;
; output: filteredMatrix: => matrix holding the filtered image
;
function ProcessRaw, directory, scanNumString, matrixSize, nImages, dcOffset
	fidFile = 1
	rotate90 = 1
	rotate180 = 2
	rotate270 = 3
	rotateTransOnly = 4
	rotateTrans90 = 5
	rotateTrans180 = 6
	rotateTrans270 = 7
	nDataPts = matrixSize[0]
	nFreqEncode = matrixSize[0]/2	; remember nFreqEncode includes RE and IM parts
	nPhaseEncode = matrixSize[1]
	reDcOffset = dcOffset[0]
	imDcOffset = dcOffset[1]
	rawData = lonarr(nDataPts*nPhaseEncode*nImages)	; rawData needs to have RE and IM parts so x2
	realDataMatrix = lonarr(nPhaseEncode,nFreqEncode)
	imDataMatrix = lonarr(nPhaseEncode,nFreqEncode)
	complexMatrix = lonarr(nPhaseEncode,nFreqEncode)
	hanningMatrix = lonarr(nPhaseEncode,nFreqEncode)
	fft1 = lonarr(nPhaseEncode, nFreqEncode)
	fft2 = lonarr(nPhaseEncode, nFreqEncode)
	image = lonarr(nPhaseEncode, nFreqEncode)

	hanningFilter = hanning(nPhaseEncode,nFreqEncode)	; define hanning filter used to filter images

	fidFileName = directory+scanNumString+'/fid'
	openr, fidFile, fidFileName, /swap_endian
	readu, fidFile, rawData		; read unformatted binary data into variable rawData
	close, fidFile
	;					  [	nFreqEncode elements    ]
	; rawData is in form: [RE|IM|RE|IM|RE|IM|RE|IM..][RE|IM|RE|IM|RE|IM..]..[       ]
	;							phase 1					phase 2			  .. phase n
	; where n = nPhaseEncode
	; So real data matrix will have size nFreqEncode x nPhaseEncode
	; And want data matrix in form:
	;	p	|...........|
	;	h	|...........|
	;	a	|...........|
	;	s	|...........|
	;	e	|...........|
	;		  f r e q
	; so keep in mind that the data MATRIX column elements refer to the frequency encoded
	; data points (or THE data acquired during the readout gradient) while the MATRIX row
	; indices refer to the corresponding phase encoding step number.
	; ALSO: Remember that IDL subscripts MATRICES _BACKWARDS_ ...i.e. not using the
	; traditional [row,column] format but using a [column,row] format
	for i = 0L, nFreqEncode - 1 do begin
		for j = 0L, nPhaseEncode - 1 do begin
			realDataMatrix[i,j] = rawData[2*i + j*nDataPts]
			imDataMatrix[i,j] = rawData[((2*i)+1) + j*nDataPts]
		endfor
	endfor

	realDataMatrix = float(realDataMatrix)
	imDataMatrix = float(imDataMatrix)
	;*********************
	; DC offset correction
	;
	realDataMatrix = realDataMatrix - reDcOffset
	imDataMatrix = imDataMatrix - imDcOffset
	;*********************

	complexMatrix = complex(realDataMatrix,imDataMatrix)

	filteredMatrix = complexMatrix; * hanningFilter
	for i = 0L, nFreqEncode - 1 do begin
		for j = 0L, nPhaseEncode - 1 do begin
			filteredMatrix[i,j] = filteredMatrix[i,j]*((-1)^(i+j))
		endfor
	endfor

	image[*,*] = abs(fft(filteredMatrix))
	live_image, rotate(image,rotateTrans90)
	return, filteredMatrix
end
