; function ImageSegment
;
; this function segments an image according to the average pixel values
; of the center of the image (60x60) and the left corner of the image (30x30).
; It is assumed that the image is centered and noise is randomly distributed
; throughout the outer 'edge' of the image.
;
;	Input: imageStats		=> structure to fill regarding various stats of image mask
;		   sigIntensities 	=> matrix holding log of signal intensity values of an image
;		   nFreqEncode 		=> # of frequency encoding steps used to acquire image(s)
;		   nPhaseEncode 	=> # of phase encoding steps used to acquire image(s)
;		   nScans			=> # of scans taken (= # of directories under scan name)
;		   boxWidth			=> # of pixels in freq encode dir used to create image mask or
;								image segment box
;		   boxHeight		=> # of pixels in phase encode dir used to create image mask or
;								image segment box
;
;	Output:
;			centerMean	=>	mean of image center pixels (boxPixels x boxPixels)
;			cornerMean	=>  mean of image left corner pixels (boxPixels/2 x boxPixels/2)
;			centerVAR	=>	variance of image center pixels
;			cornerVAR 	=> 	variance of image left corner pixels
;			centerSDV 	=>	standard deviation of image center pixels
;			cornerSDV	=>	standard deviation of image left corner pixels
;
;		*       nFreqEncode           *
;		----------------------------- *
;		|...	    				| n
;		|. . <- corners				| P
;		|...		    			| h
;		|	....................	| a
;		|	. c e n t e r s	   .	| s
;		|	....................	| E
;		|	|	boxWidth	   |	| n
;		|							| c
;		|							| o
;		|							| d
;		----------------------------- *
;
function ImageSegment, sigIntensities, nFreqEncode, nPhaseEncode, boxWidth, boxHeight


; DEBUG
print, 'nFreqEncode in ImageSegment is:'
print, nFreqEncode
print, 'nPhaseEncode in ImageSegment is:'
print, nPhaseEncode
print, ''
print, nFreqEncode/2 - boxWidth/2
print, nFreqEncode/2 + boxWidth/2
print, nPhaseEncode/2 - boxHeight/2
print, nPhaseEncode/2 + boxHeight/2
print, 'size of sigIntensities is:'
print, size(sigIntensities)
print, ''
print, ''
; DEBUG

; JUST make the corner a rectangle to cover almost the whole
; half of the image...only want to avoid the center of the image!
cornerSize = intarr(80,256)
centers = sigIntensities[0,nFreqEncode/2 - boxWidth/2:nFreqEncode/2 + boxWidth/2, $
			nPhaseEncode/2 - boxHeight/2: nPhaseEncode/2 + boxHeight/2,*]

corners = sigIntensities[0,0:boxWidth/2,(nPhaseEncode/2 - 1) - boxWidth/2:nPhaseEncode/2 - 1,*]

image_statistics, centers, MEAN=centerMean, VARIANCE=centerVar, STDDEV=centerSdv
image_statistics, corners, MEAN=cornerMean, VARIANCE=cornerVar, STDDEV=cornerSdv
imageStats = [centerMean, cornerMean, centerVar, cornerVar, centerSdv, cornerSdv]

return, imageStats
end
