; RemoveOutLyers
;
; Remove outlying voxels so T1/T2 map (or any other map)
; display is not skewed due to individual large valued voxels
; as compared to other voxels in map
;
; DGB 8/20/03
;
;	Input: 	map		=> the map whose outlyers are to be removed
;			nFreq	=> number of frequency encode steps=#columns in map
;			nPhase 	=> number of phase encode steps = # of rows in map
;			mFactor => multiplication factor. This value is multiplied
; 						by the standard deviation of the map's rows.
; 						This provides a threshold (or maximum) against which all
;						voxels in each row of the map are compared.
;		minOutLyers => minimum number of oulyers allowed in final map
;						(usually = 1)
;
;	Output: map		=> input map returned without 'outlying' voxel values
;
; 	Modification history: 6/21/04 Not sure if replacing the outlyer with the standard
; 								  deviation of the row is the best way to replace an
;								  outlyer.
pro RemoveOutLyers,map,nFreq,nPhase,mFactor,minOutLyers

	stdRows = fltarr(nPhase)
	stdRowsScaled = fltarr(nPhase)
	temp1 = intarr(nFreq,nPhase)
	temp2 = intarr(nFreq,nPhase)
	kernel = intarr(3,3)
	nKernelElements = 3*3
	kernel(*,*) = 1
	nLoops = 0
	maxLoops = 100 ; restrict while loops to max # of loops
	nOutLyers = 10000 ; this is a hack, but it works

	; loop until number of outlyers in minimized
	while nOutLyers GT minOutLyers AND nLoops LT maxLoops do begin

		; get standard deviation per row of map
		for is = 0,nPhase - 1 do begin
			stdRows[is] = stddev(map[*,is])
		endfor

		stdRowsScaled = mFactor * stdRows

		;print, size(stdRowsScaled)
		;break
		; determine where map has voxel values greater than outlyer
		; value. This comparison is done on a ROW wise basis, i.e., for the
		; is'th row, which columns have voxels that exceed the outlyer value?
		; an outlyer is considered to be a pixel whose value exceeds mFactor*standard
		; deviation of the respective row.
		for is = 0,nPhase - 1 do begin
			;print, where(map[*,is] GT outLyerDist[is])
			;print, is
			;print, nPhase
			;print, temp
			x = where(map[*,is] GT stdRowsScaled[is])
			if (x[0] NE -1) then begin 	;'where' returns -1 when test condition is false
				temp1[x,is] = 1  		; temp1 represents position of outlying pixels in image
			endif						; a 1 in the temp1 matrix represents an outlyer in the image
		endfor

		temp2 = where(temp1(*,*) GT 0)   	;rows and columns where some voxels set to 1
		colsOutLyers = temp2 MOD nFreq 		;modulo 128 gives columns with voxels < 0
									  		;MOD used here since where is passed entire matrix
		rowsOutLyers = fix(temp2 / nPhase)  ;/ 128 then rounded down gives rows with voxels <0

		; DEBUG, these two prints should produce the same result
		;print, map(where (temp1 GT 0))
		;print, map(colsOutLyers,rowsOutLyers)

		nRows = N_ELEMENTS(rowsOutLyers)
		nCols = N_ELEMENTS(colsOutLyers)
		nOutLyers = size(map(colsOutLyers,rowsOutLyers),/N_ELEMENTS)

		; First, replace outlyer with standard deviation of respective row. Then
		; take average of surrounding voxels and replace outlyer again with this
		; average. This prevents the usage of the outlyer value in the calculation
		; of the average for the current outlying value as well as for the
		; calculation of neighboring outlyer values.

		for is = 0,nOutLyers - 1 do begin

			; prevent under/over indexing of image matrix
			if (colsOutLyers[is] GT 0 AND colsOutLyers[is] LT (nPhase-1)) $
				AND  (rowsOutLyers[is] GT 0 AND rowsOutLyers[is] LT (nPhase-1)) then begin
				map[colsOutLyers[is],rowsOutLyers[is]] = stdRows[rowsOutLyers[is]]

				; grab 3x3 section around outlying voxel value
				imageKernel = map[colsOutLyers[is]-1:colsOutLyers[is]+1,$
									rowsOutLyers[is]-1:rowsOutLyers[is]+1]

				map[colsOutLyers[is],rowsOutLyers[is]] = total(imageKernel)/nKernelElements

			endif else begin
				map[colsOutLyers[is],rowsOutLyers[is]] = stdRows[rowsOutLyers[is]]
			endelse
		endfor

	nLoops = nLoops + 1
	;live_image, map,TITLE='t2MapLin2 with outlyers replaced by std'
	endwhile
end
