; pro CalcNoiseFloor
; NOTE that CalcNoiseFloor is searching for the edges of the image using
; t statistics between rows and columns.
; Determine a noise floor of an image by looking at how voxel
; values differ from one another per row...then choose the row
; with highest significant difference (use standard deviation as
; statistical measure). Compare values in that row to determine S/N
;
;	Input: noiseFloor	=> (to be overwritten and returned) 'floor' or level at which
;							noise ends and signal begins.
;		   noiseFactor	=> this allows the user to increase or decrease the noise
;						   floor by a multiplicative factor
;		   nFreq		=> # frequency encode steps = # columns in images
;		   nPhase 		=> # phase encode steps = # rows in images
;		   image		=> image acquired on Bruker (or other scanner).
;		   qValue		=> the statistical 'q-value'. Used to determine the sigificance
;						   of the t-test result
;
;	Output: noiseFloor	=> 'floor' or level at which noise ends and signal begins.
;
;	Modification History: Created 8/27/03 dgb
;							10/28/03: changed images to image since only one
; 									  image was being used in code anyway

pro CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, image, qValue

	stdImage = fltarr(nPhase)
	tStatsRows = fltarr(2,nPhase) ; tm_test returns [t_stat, significance]
	tStatsCols = fltarr(nFreq,2)
	rowsNoise = intarr(2)
	colsNoise = intarr(2)

	; calculate the standard deviation of the values
	; within each row of an image.
	for is = 0,nPhase - 1 do begin
		stdImage(is) = stddev(image(*,is)) ; use the 0th image for now
	endfor

	; get t statistics of image row-wise
	for is = 0,nPhase - 2 do begin
			; check that rows to be t tested are not zero filled
			if ((total(image(*,is)) EQ 0) AND (total(image(*,is+1)) EQ 0)) then begin
				print, 'Found zero filled rows'
				;tStatsRows(*,is) = 10.0 ;try this and see what happens! 10/28/03
			endif else begin
				tStatsRows(*,is) = tm_test(image(*,is),image(*,is+1),/UNEQUAL)
			endelse
	endfor

	nColsZeroFilled = 0
	; get t statistics of image column-wise
	for is = 0,nFreq - 2 do begin
			; first check that rows and columns are not zero filled before running
			; t test
				if ((total(image(is,*)) EQ 0) AND (total(image(is+1,*)) EQ 0)) then begin
					nColsZeroFilled = nColsZeroFilled + 1
					;tStatsCols(is,*) = 10.0
				endif else begin
					tStatsCols(is,*) = tm_test(image(is,*),image(is+1,*),/UNEQUAL)
				endelse
	endfor

	; DEBUG
	;print, nColsZeroFilled

	; the last element is always ?? 0.00000 so don't include it in the
	; comparison
	rowsNoise =  where(tStatsRows(1,0:nPhase-2) LT qValue)
	colsNoise =  where(tStatsCols(0:nFreq-2,1) LT qValue)

	; DEBUG
	;print, tStatsRows
	;print, 'BREAK'
	;print, tStatsCols
	;print, 'BREAK'
	;print, rowsNoise
	;print, 'BREAK'
	;print, colsNoise
	;DEBUG

	nRowsNoise = N_ELEMENTS(rowsNoise)
	nColsNoise = N_ELEMENTS(colsNoise)

	; updated 11/03/03
	; find the max value within the region defined by the
	; noisy rows and noisy columns only
	;noiseFloor = max(image(colsNoise[0]:colsNoise[nColsNoise - 1],rowsNoise[0]:rowsNoise[nRowsNoise - 1]))
	noiseFloor = mean(image(colsNoise[0]:colsNoise[nColsNoise - 1],rowsNoise[0]:rowsNoise[nRowsNoise - 1]))
	; select voxels outside region defined by edges
	; of image (which were determined using t statistics)
	; take mean of these voxels and use as noise floor
	;a = max(image(0:colsNoise[0],0:rowsNoise[0]))
	;b = max(image(colsNoise[1]:nFreq-1,rowsNoise[1]:nPhase-1))
	;c = max(a,b)
	;print, c

	; OK here's the noise floor, let's see how it works
	noiseFloor = noiseFactor*noiseFloor
end
