; NonLinMap
;
;	Inputs: t2MapNonLin	=> nonlinear fitted t2 map to be returned
;			m0MapNonLin	=> nonlinear fitted m0 map to be returned
;			t2MapLin	=> average used for initial guess (T2) for curvefit routine
;			m0MapLin 	=> average used for initial guess (m0) for curvefit
;			mask		=> noise mask (image showing 'noisy' pixels only)
;			images		=> actual images acquired on Bruker (or other scanner)
;			nFreq		=> # frequency encode steps = # columns in images
;			nPhase 		=> # phase encode steps = # rows in images
;			noise		=> noise floor
;			nImages		=> # images in 'images'
;			TE			=> Vector of echo times to fit against
;
;	Outputs:t2MapNonLin	=> yes, this is an output also. nonlinear fitted t2 map
;			m0MapNonLin	=> nonlinear fitted m0 map.

;	average value of m0MapLin and t2MapLin matrix is used as initial guess
;	of m0,t2 values for non-linear curve fitting routine.
;	When calculating the average value of m0,t2, use only those pixels
;	that are not noisy. The # of pixels not noisy = # of pixels
;	in mask matrix that are equal to 1 (or sum of pixels in
;	noisePixels matrix)
;
;	Modification History: Created 8/20/03
;						  8/27/03: add t2MapLin,m0MapLin,mask arguments
;
pro NonLinMap,t2MapNonLin,m0MapNonLin,t2MapLin,m0MapLin,mask,images,nFreq,nPhase,noise,nImages,TE

	A = fltarr(2)				; store parameters of fitted curve
	yfit = fltarr(2)
	W = fltarr(nImages)
	m0FitGuess = total(m0MapLin)/total(mask)
	t2FitGuess = total(t2MapLin)/total(mask)
	noiseFloor = indgen(nImages)
	noiseFloor[*] = noise
	A = [30000,.5]	; provide initial guess of non-linear parameters
	W(*) = 1.0
	converge = 1 				; assume convergence
	nPixelsMapped = 0
	nNonConverge = 0

	for i = 0,nFreq - 1 do begin
		for j = 0,nPhase - 1 do begin
			; compare current pixel in each image to noise threshold.
			; if pixel value is less than noise value, reject the pixel
			; across all images
			; after looking at Zhaohui's data, first image (first TE) does not look
			; 'right', i.e., first image is less intense than second image (second TE)
			goodPixels = where(images[i,j,1:31] GT noiseFloor[*])
			;print, goodPixels
			; fit may be done across all images, or only 3 (at least 3 TE's and 3 images.
			; Curvefit requires at least 3 data points for the fit)
			if (goodPixels[0] NE -1) AND (N_ELEMENTS(goodPixels) GT 2) then begin
				yfit = DBCurvefit(TE[goodPixels],transpose(images[i,j,goodPixels+1]),$
				W,A,SIGMA,FUNCTION_NAME = 'T2curveFitFunc',CONVERGENCE=converge )

				;if curvefit does not converge reset A (guess). If at least one pixel has already
				;been mapped, reset the parameter guess (A) values to the average of the
				;values that have already been mapped. This provides curvefit with a
				;good guess for the next iteration.
				if (converge EQ 0) OR (converge EQ -1) then begin	; do NOT map the current values in A!

					if nPixelsMapped GT 1 then begin
						A[1] = total(t2MapNonLin)/nPixelsMapped ; A[1] = average of t2 map for next iteration
						A[0] = total(m0MapNonLin)/nPixelsMapped	; A[0] = average of m0 map for next iteration
					endif

				endif else begin
					; curvefit converged so we can map the current pixel's t2 and m0 value
					nPixelsMapped = nPixelsMapped + 1
					t2MapNonLin[i,j] = A[1]
					m0MapNonLin[i,j] = A[0]
				endelse

			endif
		endfor
	endfor

end





