; RemoveNegPixels
;
;Input: 	image 	=> image or map whose negative pixels are removed
;			nFreq	=> # of freq encode steps = # columns of image matrix
;			nPhase	=> # of phase encode steps = # rows of image matrix
;
; created: 8/13/03
;
; update history:	8/21/03: replace neg pixel value with std of row then
;							 average of each surrounding pixel including
;							 replaced value (3x3 average)
;
; remember, IDL updates 'image' and returns the
; updated version of image to the caller.
;
;
pro RemoveNegPixels, image,nFreq,nPhase

	stdRows = fltarr(nPhase)

	temp = where(image(*,*) LT 0)   ;rows and columns where some pixels < 0
	if (temp[0] EQ -1) then begin
		print, 'No negative pixels found'
		return
	endif

	colsNeg = temp MOD nFreq		;test modulo 128 gives columns with pixels < 0
									; MOD used here since where is passed entire matrix
	rowsNeg = fix(temp / nPhase)	;test / 128 then rounded down gives rows with pixels <0
		;****** DEBUG
		;print, 'In RemoveNegPixels'
		;print, temp
		;print, image(temp)
		;print, image[colsNeg,rowsNeg]
		;print, colsNeg
		;print, rowsNeg
		;break

	nColsNeg = N_ELEMENTS(colsNeg)
	nRowsNeg = N_ELEMENTS(rowsNeg)

	nNegPixels = size(image[colsNeg,rowsNeg],/N_ELEMENTS)
	kernel = intarr(3,3)
	nKernelElements = 3*3
	kernel(*,*) = 1

	; get standard deviation per row of image
	for is = 0,nPhase - 1 do begin
		stdRows[is] = stddev(image[*,is])
	endfor

	; First, replace neg pixel with standard deviation of respective row.
	; Then take average of surrounding pixels and replace neg pixel again
	; with this average. This prevents the usage of neg value in the
	; calculation of the average for the current neg value as well as for
	; the calculation of neighboring neg values.
	; don't go through the entire image. Only look at those columns and rows
	; that have pixel values < 0
   	for is = 0,nNegPixels - 1 do begin
   			image[colsNeg[is],rowsNeg[is]] = stdRows[rowsNeg[is]]

			; prevent under/over indexing of image matrix
			if (colsNeg[is] GT 0 AND rowsNeg[is] GT 0) AND $
			   (colsNeg[is] LT (nPhase-1) AND rowsNeg[is] LT (nPhase-1)) then begin

				;print, colsNeg[is]
				;print, rowsNeg[is]
				;print, image[0:2,0:2]
				imageKernel = image[colsNeg[is]-1:colsNeg[is]+1,$
								rowsNeg[is]-1:rowsNeg[is]+1] ; 3x3 section around neg. value

				;***** DEBUG check for negative elements within kernel since
				;print, imageKernel
				;print, '******************'

				average = total(imageKernel)/nKernelElements

			endif else begin ; or just use standard deviation for average
				average = stdRows[rowsNeg[is]]
			endelse

			; replace the current pixel value with the average of surrounding
			; values (1 pixel surround) if average is > 0
			if average GT 0 then begin ; don't replace a neg pixel with a neg average!
				image[colsNeg[is],rowsNeg[is]] = average
			endif else begin	;can't get a pos. average so just replace with zero
			 	image[colsNeg[is],rowsNeg[is]] = 0
			endelse

	endfor

end