; pro CalcNoiseFloor
; NOTE that CalcNoiseFloor is searching for the edges of the image using
; t statistics between rows and columns.
; Determine a noise floor of an image by looking at how voxel
; values differ from one another per row...then choose the row
; with highest significant difference (use standard deviation as
; statistical measure). Compare values in that row to determine S/N
;
;	Input: noiseFloor	=> (to be overwritten and returned) 'floor' or level at which
;							noise ends and signal begins.
;		   noiseFactor	=> this allows the user to increase or decrease the noise
;						   floor by a multiplicative factor
;		   nFreq		=> # frequency encode steps = # columns in images
;		   nPhase 		=> # phase encode steps = # rows in images
;		   image		=> image acquired on Bruker (or other scanner).
;		   qValue		=> the statistical 'q-value'. Used to determine the sigificance
;						   of the t-test result
;
;	Output: noiseFloor	=> 'floor' or level at which noise ends and signal begins.
;
;	Modification History: Created 8/27/03 dgb
;							10/28/03: changed images to image since only one
; 									  image was being used in code anyway
;							11/03/03: noiseFloor = mean value within the region defined by
; 									  the noisy rows and noisy columns only.
;							6/21/04: Update comments regarding tm_test routine
;							6/22/04: Compare mean of previous 5 pixels to next 5 pixels. If
;							mean is significantly greater (student's t-test) the current pixel
;							is an edge pixel

pro CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, images, qValue

	DEBUG_1 = 0
	DEBUG_2 = 0

	nBackPixels = 10 ;# of pixels before the current pixel to use in t-test
	nForwardPixels = 2 ; # of pixels after the current pixel to use in t-test
	zeroNoiseBound = intarr(nFreq,nPhase)
	noiseSkinBound = intarr(nFreq,nPhase)
	skullBrainBound = intarr(nFreq,nPhase)

	stdImage = fltarr(nPhase)
	; tm_test: [t_stat, significance]
	; tStats:  [ [t_stat,sig],[t_stat,sig],[t_stat,sig],...] => vectorSize row
	;			    [t_stat,sig],[t_stat,sig],[t_stat,sig],...] => vectorSize+1 row
	;			    [t_stat,sig],[t_stat,sig],[t_stat,sig],...] => (nPhase - 1) row
    ; NOTE:
    ;     tStatsCol gives the columns that exist in the noise area of the image

	tStats = fltarr(2*((nFreq-1)-(2*nBackPixels)),nPhase,2)

	; tStatsRow: [[t_stat,sig] => for 'nBackPixels' pixels before and 2 after 5th
	;			  [t_stat,sig] => for 'nBackPixels' pixels before and 2 after 6th
	;			  [t_stat,sig] => " ..							7th
	;
	;						...]	   => " ..   			nPhase - 'nBackPixels'th row
	;NOTE:
    ;     tStatsRow gives the rows that exist in the noise area of the image

	;tStatsRow = fltarr((nFreq-1),2*((nPhase-1)-(2*nBackPixels)),2)

	;;;; DEBUG
	;print, size(tStatsRow)
	;print, size(tStatsCol)
	;;

	;rowsNoise = intarr(2)
	;colsNoise = intarr(2)
	;temp = intarr(nFreq,nPhase)

	nImages = N_ELEMENTS(images(*,0,0))

	; Start the main for loop to loop through each image in
	; 'images' and find the noise floor for each image
	for imageNum = 0, nImages-1 do begin

		; get t statistics of image based on 5 pixel vector. For example, compare,
		; using the student's t-test the mean of the previous 5 pixels to the mean of
		; the next 5 pixels. tm_test returns the t-statistic for the comparison.
		; The significance is a value in the interval [0.0, 1.0]; A small value
		; (0.05 or 0.01) indicates that row 'is' and 'is+1' have significantly different
		; means

		for row = 0,nPhase-1 do begin
			for col = nBackPixels,(nFreq-1) - nBackPixels do begin
				; check vector (before and after current pixel) to be t tested is not 0 filled
				if ((total(images(imageNum,col-nBackPixels:col-1,row)) EQ 0) AND $
				(total(images(imageNum,col+1:col+nForwardPixels,row)) EQ 0)) then begin
					;print, 'Found zero filled vectors while in CalcNoiseFloor.pro, skip to next pixel'
				endif else begin

				; NOTE that tStatsRow starts at ('nBackPixels',0). AND, DONT include the current pixel
				; in the t-test
				tStats[col,row,*] = tm_test(images[imageNum,col-nBackPixels:col-1,row],$
				images[imageNum,col+1:col+nForwardPixels,row]);,/UNEQUAL)
				endelse

				;********************************************************************************************
				; D E B U G
				if (DEBUG_1 EQ 1) then begin
					if (row EQ 1 AND col EQ (nFreq - 1 - nBackPixels) AND imageNum EQ 0) then begin
						for rowTemp = 0, 1 do begin
							for colTemp = 60, 120 do begin
								print, '******************************************************************************'
								print, FORMAT = '("CalcNoiseFloor, Row #",I," Col #",I)',rowTemp,colTemp
								print, FORMAT = '("Comparing these pixel values: ",("[",10I,"]"), " To these: ",("[",2I,"]") )', $
								images[imageNum,colTemp-nBackPixels:colTemp-1,rowTemp],$
								images[imageNum,colTemp+1:colTemp+nForwardPixels,rowTemp]
								print, FORMAT = '("The t-test results are: ",2F)',tStats[colTemp,rowTemp,*]
								print, '*******************************************************************************'
							endfor
						endfor
					endif
				endif
				; E N D  D E B U G
				;********************************************************************************************
	    	endfor ; for col =

	    endfor ; for row =

		; want to find t-test significance factors that are > 0.00000 and < 0.001
		; the first value that's in this range is the zero-fill/noise boundary.
		; the second value in this range is the noise/skin boundary
		; third value in this range is the skull/brain boundary (??).

		for tStatRow = 0, nPhase - 1 do begin
			; thresholdColumns contains the columns of tStats that have pixels
			; whose t-test significance value is in the range: 0 < sig < 0.001
			thresholdCols = where(tStats(*,tStatRow,1) LT qValue AND tStats(*,tStatRow,1) GT 0)
			nThresholdCols = N_ELEMENTS(thresholdCols)

			if (thresholdCols[0] NE -1) then begin
				zeroNoiseBound[thresholdCols[0],tStatRow] = 1

				;********************************************************************************************
				; D E B U G
				if (DEBUG_2 EQ 1) then begin
					print, FORMAT = '("CalcNoiseFloor, found zero/noise boundary pixel at Row # ",I," Col # ",I)'$
					,tStatRow,thresholdCols[0]
					print, thresholdCols
					DAVE
				endif
				; E N D  D E B U G
				;********************************************************************************************

				if (nThresholdCols GT 1) then begin
					noiseSkinBound[thresholdCols[1],tStatRow] = 1

					;********************************************************************************************
					; D E B U G
					if (DEBUG_2 EQ 1) then begin
						print, FORMAT = '("CalcNoiseFloor, found noise/skin boundary pixel at Row # ",I," Col # ",I)'$
						,tStatRow,thresholdCols[1]
					; E N D  D E B U G
					;********************************************************************************************
					endif
				endif ; if (nThresholdCols GT 1...)
			endif ; if (thresholdCols[0] NE -1)...


			; now try to find the skull/brain interface. Can't assume that the column
			; for this boundary is the next element of thresholdCols. Due to the -10/+2 method
			; of generating the t-test significance values. Here's an example of one row of
			; thresholdCols matrix:
			; [35 83 84 85 86 95 109 110 111 112]
			; The first element, 35, represents the column number that, together with the current row,
			; tStatRow, represents the pixel on the zero-fill/noise boundary. The jump from 35 to 83
			; represents the noise/skin boundary. The jump from 86 to 95 represents the skull/brain
			; boundary. We can now use these values to remove the noise, skull, or both.

			; start at the second element = '1'
			for is = 1, nThresholdCols do begin
				; make sure we have at least 2 elements in thresholdCols vector
				if (nThresholdCols GT 2) then begin
					if ((thresholdCols[is+1] + 5) GT thresholdCols[is]) then begin
						skullBrainBound[thresholdCols[is+1],tStatRow] = 1
						break ; ok, found the column we need, quit out of for loop
					endif
				endif
			endfor

		endfor ;for tStatRow = 0, nPhase - 1

		;;;; DEBUG
		live_image, zeroNoiseBound, TITLE = 'zero-fill / noise boundary
		live_image, noiseSkinBound, TITLE = 'noise / skin boundary'
		live_image, skullBrainBound, TITLE = 'skull / brain boundary'
		DAVE
		;testImage = fltarr(256,256)

		;colsNoise = noisePixels MOD nFreq 		;modulo 128 gives columns with voxels < 0
									  		;MOD used here since where is passed entire matrix
		;rowsNoise = fix(noisePixels / nPhase)  ;/ 128 then rounded down gives rows with voxels <
		;testImage(colsNoise,rowsNoise) = 1
		;live_image, testImage

		;print, size(tStats)
		;print, size(noise)
		;print, noise

		;;;; DEBUG
		;print, tStatsRow
		DAVE
		;;;;

		;;;; DEBUG
		;print, nColsZeroFilled
		;print, tStatsRows

		;;;;

		; the last element is always ?? 0.00000 so don't include it in the
		; comparison
		;rowsNoise =  where(tStatsRows(1,0:nPhase-2) LT qValue)
		;colsNoise =  where(tStatsCols(0:nFreq-2,1) LT qValue)

		; DEBUG
		;print, tStatsRows
		;print, 'BREAK'
		;print, tStatsCols
		;print, 'BREAK'
		;print, rowsNoise
		;print, 'BREAK'
		;print, colsNoise
		;DEBUG

		;nRowsNoise = N_ELEMENTS(rowsNoise)
		;nColsNoise = N_ELEMENTS(colsNoise)

		; find the max value within the region defined by the
		; noisy rows and noisy columns only
		;noiseFloor = max(image(colsNoise[0]:colsNoise[nColsNoise - 1],rowsNoise[0]:rowsNoise[nRowsNoise - 1]))

		;noiseFloor(imageNum) = mean(images(imageNum,colsNoise[0]:colsNoise[nColsNoise - 1],$
		;rowsNoise[0]:rowsNoise[nRowsNoise - 1]))

		; select voxels outside region defined by edges
		; of image (which were determined using t statistics)
		; take mean of these voxels and use as noise floor
		;a = max(image(0:colsNoise[0],0:rowsNoise[0]))
		;b = max(image(colsNoise[1]:nFreq-1,rowsNoise[1]:nPhase-1))
		;c = max(a,b)
		;print, c

		; OK here's the noise floor, let's see how it works
		;noiseFloor = noiseFactor*noiseFloor(*)

	endfor ; for imageNum = 0,

end
