; pro CalcNoiseFloor
; NOTE that CalcNoiseFloor is searching for the edges of the image using
; t statistics between rows and columns.
; Determine a noise floor of an image by looking at how voxel
; values differ from one another per row...then choose the row
; with highest significant difference (use standard deviation as
; statistical measure). Compare values in that row to determine S/N
;
;	Input: noiseFloor	=> (to be overwritten and returned) 'floor' or level at which
;							noise ends and signal begins.
;		   noiseFactor	=> this allows the user to increase or decrease the noise
;						   floor by a multiplicative factor
;		   nFreq		=> # frequency encode steps = # columns in images
;		   nPhase 		=> # phase encode steps = # rows in images
;		   image		=> image acquired on Bruker (or other scanner).
;		   qValue		=> the statistical 'q-value'. Used to determine the sigificance
;						   of the t-test result
;
;	Output: noiseFloor	=> 'floor' or level at which noise ends and signal begins.
;
;	Modification History: Created 8/27/03 dgb
;							10/28/03: changed images to image since only one
; 									  image was being used in code anyway
;							11/03/03: noiseFloor = mean value within the region defined by
; 									  the noisy rows and noisy columns only.
;							6/21/04: Update comments regarding tm_test routine

pro CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, images, qValue

	stdImage = fltarr(nPhase)
	tStatsRows = fltarr(2,nPhase) ; tm_test returns [t_stat, significance]
	tStatsCols = fltarr(nFreq,2)
	rowsNoise = intarr(2)
	colsNoise = intarr(2)
	nImages = N_ELEMENTS(images(*,0,0))
	temp = intarr(nFreq,nPhase)

	; Start the main for loop to loop through each image in
	; 'images' and find the noise floor for each image
	for imageNum = 0, nImages-1 do begin

		; calculate the standard deviation of the values
		; within each row of an image.
		for is = 0,nPhase - 1 do begin
			stdImage(is) = stddev(images(imageNum,*,is))
		endfor

		; get t statistics of image row-wise. For example, compare, using the student's
		; t-test the mean of the first row to the mean of the second row. tm_test returns
		; the t-statistic for the comparison. The significance is a value in the interval
		; [0.0, 1.0]; A small value (0.05 or 0.01) indicates that row 'is' and 'is+1' have
		; significantly different means
		for is = 0,nPhase - 2 do begin
				; check that rows to be t tested are not zero filled
				if ((total(images(imageNum,*,is)) EQ 0) AND (total(images(imageNum,*,is+1)) EQ 0)) then begin
					print, 'Found zero filled rows while in CalcNoiseFloor.pro, skip to next row'
				endif else begin
					tStatsRows(*,is) = tm_test(images(imageNum,*,is),images(imageNum,*,is+1),/UNEQUAL)
				endelse

				if (is LT 30) then begin
					print, '**********************************************'
					print, FORMAT = '("CalcNoiseFloor, Row # ",I)',is
					print, transpose(images(imageNum,0:160,is))
					print, '**********************************************'

				endif
				if (imageNum GT 0 ) then BREAK

	    endfor

		nColsZeroFilled = 0
		; get t statistics of image column-wise
		for is = 0,nFreq - 2 do begin
			; 1st check that rows and columns are not zero filled b4 running t test
				if ((total(images(imageNum,is,*)) EQ 0) AND (total(images(imageNum,is+1,*)) EQ 0)) then begin
					nColsZeroFilled = nColsZeroFilled + 1
				endif else begin
					; Calculate the t-statistic of the current
					tStatsCols(is,*) = tm_test(images(imageNum,is,*),images(imageNum,is+1,*),/UNEQUAL)
				endelse
		endfor

		;;;; DEBUG
		;print, nColsZeroFilled
		;print, tStatsRows

		;;;;

		; the last element is always ?? 0.00000 so don't include it in the
		; comparison
		rowsNoise =  where(tStatsRows(1,0:nPhase-2) LT qValue)
		colsNoise =  where(tStatsCols(0:nFreq-2,1) LT qValue)

		; DEBUG
		;print, tStatsRows
		;print, 'BREAK'
		;print, tStatsCols
		;print, 'BREAK'
		print, rowsNoise
		print, 'BREAK'
		print, colsNoise
		;DEBUG

		nRowsNoise = N_ELEMENTS(rowsNoise)
		nColsNoise = N_ELEMENTS(colsNoise)

		; find the max value within the region defined by the
		; noisy rows and noisy columns only
		;noiseFloor = max(image(colsNoise[0]:colsNoise[nColsNoise - 1],rowsNoise[0]:rowsNoise[nRowsNoise - 1]))
		noiseFloor(imageNum) = mean(images(imageNum,colsNoise[0]:colsNoise[nColsNoise - 1],$
		rowsNoise[0]:rowsNoise[nRowsNoise - 1]))
		; select voxels outside region defined by edges
		; of image (which were determined using t statistics)
		; take mean of these voxels and use as noise floor
		;a = max(image(0:colsNoise[0],0:rowsNoise[0]))
		;b = max(image(colsNoise[1]:nFreq-1,rowsNoise[1]:nPhase-1))
		;c = max(a,b)
		;print, c

		; OK here's the noise floor, let's see how it works
		noiseFloor = noiseFactor*noiseFloor(*)

	endfor ; for imageNum = 0,

end
